/*--

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 1999  Microsoft Corporation.  All rights reserved.

Module Name:

    computer.c

Abstract:

    Sample program that demonstrates how to create a computer account
    using LDAP.

--*/

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <netdb.h>
#include <netinet/in.h>
#ifdef HAVE_UNAME
#include <sys/utsname.h>
#endif
#include <krb5.h>
#include "lber.h"
#include "ldap.h"
#include "sam.h"
#include "computer.h"

static const char rcsid[] = "$Id$";

int find_host(
    LDAP *ld,
    LDAPMessage **res,
    char *bind_path,
    char *host
    )
{
    char search_exp[1024];

    sprintf(search_exp, "(samAccountName=%s$)", host);
    return ldap_search_s(ld, bind_path, LDAP_SCOPE_SUBTREE,
			 search_exp, NULL, 0, res);
}

int add_host(
    LDAP *ld,
    char *bind_path,
    char *host,
    char *hostname,
    char *domainname,
    int   thishost
    )
{
#define ADD_ATTR(t, v) 		\
    mods[n] = malloc(sizeof(LDAPMod));	\
    mods[n]->mod_op = LDAP_MOD_ADD;	\
    mods[n]->mod_type = t; 		\
    mods[n++]->mod_values = v

#ifdef HAVE_UNAME
    struct utsname utsname;
#endif
    LDAPMod *mods[10];
    char new_dn[256], *host_spn, *host_upn, *cp;
    char *operatingSystem = "";
    char *operatingSystemVersion = "";
    char *cn_v[] = {NULL, NULL};
    char *objectClass_v[] = {
	"top", "person", "organizationalPerson", "user", "computer", NULL 
    };
    char *dnsHostName_v[] = {NULL, NULL};
    char *operatingSystem_v[] = {NULL, NULL};
    char *operatingSystemVersion_v[] = {NULL, NULL};
    char *samAccountName = NULL;
    char *samAccountName_v[] = {NULL, NULL};
    char *servicePrincipalName_v[] = {NULL, NULL};
    char *userPrincipalName_v[] = {NULL, NULL};
    char *userAccountControl_v[] = {NULL, NULL};
    char userAccountControlStr[80];
    u_int userAccountControl =
	UF_DONT_EXPIRE_PASSWD | UF_WORKSTATION_TRUST_ACCOUNT |
	UF_TRUSTED_FOR_DELEGATION | UF_USE_DES_KEY_ONLY;
    int n;
    
    host_spn = malloc(strlen(hostname)+6);
    sprintf(host_spn, "host/%s", hostname);
    host_upn = malloc(strlen(domainname)+strlen(host_spn)+2);
    for (cp = domainname; *cp; cp++)
	*cp = toupper(*cp);
    
    sprintf(host_upn, "%s@%s", host_spn, domainname);

    dnsHostName_v[0] = hostname;
    
#ifdef HAVE_UNAME
    if (thishost) {
	uname(&utsname);
	operatingSystem = malloc(strlen(utsname.sysname)+strlen(utsname.machine)+10);
	sprintf(operatingSystem, "%s/%s", utsname.sysname, utsname.machine);
	operatingSystemVersion = strdup(utsname.version);
    }
#else
    operatingSystem = "Unknown";
    operatingSystemVersion = "Unknown";
#endif
    
    sprintf(new_dn, "cn=%s,%s", host, bind_path);
    cn_v[0] = host;
    samAccountName = malloc(strlen(host)+2);
    sprintf(samAccountName, "%s$", host);
    samAccountName_v[0] = samAccountName;
    
    sprintf(userAccountControlStr, "%ld", userAccountControl);
    userAccountControl_v[0] = userAccountControlStr;

    if (thishost) {
	operatingSystem_v[0] = operatingSystem;
	operatingSystemVersion_v[0] = operatingSystemVersion;
    }
    servicePrincipalName_v[0] = host_spn;
    userPrincipalName_v[0] = host_upn;
    
    n = 0;
    ADD_ATTR("cn", cn_v);
    ADD_ATTR("sAMAccountName", samAccountName_v);
    ADD_ATTR("objectClass", objectClass_v);
    ADD_ATTR("userPrincipalName", userPrincipalName_v);
    ADD_ATTR("servicePrincipalName", servicePrincipalName_v);
    ADD_ATTR("dNSHostName", dnsHostName_v);
    ADD_ATTR("userAccountControl", userAccountControl_v);
    if (thishost) {
	ADD_ATTR("operatingSystem", operatingSystem_v);
	ADD_ATTR("operatingSystemVersion", operatingSystemVersion_v);
    }
    mods[n] = NULL;

    return ldap_add_ext_s(ld, new_dn, mods, NULL, NULL);
}
