.\" 
.\" THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
.\" ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
.\" TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
.\" PARTICULAR PURPOSE.
.\" 
.\" Copyright (C) 1999  Microsoft Corporation.  All rights reserved.
.\" 
.TH ADDUSER 1m \*h
.SH NAME
adduser \- Create, delete, or update a user's properties using an LDAP update to Active Directory
.SH SYNOPSIS
.B adduser
[-A admin-account] [-D domain] [-O ous] [-s LDAPserver] [-p LDAPport] [-B bind-path] [-a attributeName=value] [-vI]
[\fIuser@domain\fP]
.PP
.B adduser
-x [-A admin-account] [-D domain] [-O ous] [-s LDAPserver] [-p LDAPport] [-B bind-path] [-v]
[\fIuser@domain\fP]
.SH DESCRIPTION
.PP
.B Creating New Users
.PP
When invoked without the \fI\-x\fP option,  the  adduser  command
creates  a  new user account using the values specified on
the command line and the default values from  the  system. If the directory
schema has been extended with the Services for Unix Posix schema, appropriate
entries from the corresponding local passwd entry will be added to the
account.
.PP
.B Deleting Users
.PP
When invoked with the \fI\-x\fP option,  the  adduser  command
deletes a user account using the values specified on
the command line and the default values from  the  system.

.SH OPTIONS
.TP
\fB\-A\fP \fIadmin-account\fP
Specifies simple LDAP bind as the DN based admin-account identity.
.TP
\fB\-D\fP \fIdomain\fP
Specifies the LDAP domain to bind to.  This option is unnecessary
if the domain is specified in <user@domain>.
.TP
\fB\-O\fP \fIOU\fP
Specifies which organizational unit (OU) to create accounts in. (default is "cn=users")
.TP
\fB\-s\fP \fILDAPserver\fP
Specifies hostname of LDAP host to bind to.  This option
is usually unnecessary because the ldap_host is found
given the domain name, automatically, by DNS lookup.
.TP
\fB\-p\fP \fILDAPport\fP
Specifies LDAP port to bind to.  Defaults to LDAP_PORT
which is the standard LDAP bind port (389).
.TP
\fB\-B\fB \fIbind-path\fP
Specifies LDAP path to bind to.  Defaults to the 
entire domain based DN.
.TP
\fB\-a\fB \fIattributeName=value\fP
Adds this string attribute to a multi-value property
or replaces a previous single-value property.  This option
can be invoked serveral times to add attributes to the same
property or attributes to multiple property.
.TP
\fB\-v\fB
Enables verbose output.
.TP
\fB\-I\fB
Invalidates account for logon.  This option is currently
only enabled upon account creation.

.SH RETURN VALUES
.TP
\fB0\fB	SUCCESS:
.TP
\fBadd user:\fP
The user was found and attributes were successfully updated or the
user was not found and account was successfully created.
.PP

\fBdelete user:\fP
The user was found and successfully deleted or the
user was not found
.TP
\fB1\fB	FAILURE:
.TP

\fBadd user:\fP
The user was found and attributes were not updated. Or the
user was not found but account was not created
.TP
\fBdelete user:\fP
The user was found but not deleted due to insufficient access.
.TP
\fBsimple bind:\fP
reading the user's password failed.
.TP
\fBgeneral:\fP
The LDAP bind failed. The
LDAP find user failed. Or the
set_password() failed.
.SH SEE ALSO
ksetpw(1m)
