/*
 * Copyright (C) 1998-2000 Luke Howard. All rights reserved.
 * CONFIDENTIAL
 * $Id$
 *
 * General GSS-API SASL (not LDAP specific) stuff.
 */

#ifndef _GSSSASL_H_
#define _GSSSASL_H_

#ifdef HEIMDAL
#include <gssapi.h>
#else
#include <gssapi/gssapi_generic.h>
#endif
#include <netinet/in.h>

#define GSSAPI_SASL_NAME                  "GSSAPI"

/* GSS_Accept_Context */
#define GSSSASL_NEGOTIATE_CONTEXT         0x01
/* Section 7.2.2 of RFC 2222 */
#define GSSSASL_SEND_SECURITY_OPTIONS     0x02
/* Section 7.2.1 of RFC 2222 */
#define GSSSASL_RECEIVE_SECURITY_OPTIONS  0x03
#define GSSSASL_COMPLETED                 0x04

/* Section 7.3 of RFC 2222 */
#define GSSSASL_NO_SECURITY_LAYER         0x01
#define GSSSASL_INTEGRITY_PROTECTION      0x02
#define GSSSASL_PRIVACY_PROTECTION        0x04

/* Token for security negotiation (after GSS-API nego) */
typedef struct gsssasl_security_negotiation_desc {
    /* Section 7.2.3 of RFC 2222 */
    int security_layer:8;
    /* Section 7.2.[12] of RFC 2222 */
    int token_size:24;
} gsssasl_security_negotiation_desc, *gsssasl_security_negotiation_t;

/* Section 7.2.1 of RFC 2222 */
/*    identity follows gsssasl_security_negotiation_desc */

typedef int gsssasl_server_negotiation_desc, *gsssasl_server_negotiation_t;

/* Wrap and encode a security negotiation token */
static inline OM_uint32 gsssasl_pack_security_token(OM_uint32 * min_stat,
						    gss_ctx_id_t context,
				   gsssasl_security_negotiation_t inmask,
						    size_t masklength,
						gss_buffer_t wrapped_tok)
{
    OM_uint32 maj_stat, rc;
    gss_buffer_desc send_tok;

    wrapped_tok->length = 0;
    wrapped_tok->value = NULL;

    if (masklength < sizeof(gsssasl_security_negotiation_desc))
	return GSS_S_FAILURE;

    inmask->token_size = htonl(inmask->token_size);

    send_tok.length = masklength;
    send_tok.value = inmask;

    maj_stat = gss_wrap(min_stat,
			context,
			0,
			GSS_C_QOP_DEFAULT,
			&send_tok,
			&rc,
			wrapped_tok);

    inmask->token_size = ntohl(inmask->token_size);

    return maj_stat;
}

/* Unwrap and decode a security negotiation token. */
static inline int gsssasl_unpack_security_token(OM_uint32 * min_stat,
						gss_ctx_id_t context,
						gss_buffer_t wrapped_tok,
				gsssasl_security_negotiation_t * outmask,
						size_t * masklength)
{
    OM_uint32 maj_stat, rc;
    gss_buffer_desc recv_tok;

    *masklength = 0;
    *outmask = NULL;

    maj_stat = gss_unwrap(min_stat,
			  context,
			  wrapped_tok,
			  &recv_tok,
			  &rc,
			  (gss_qop_t *) NULL);
    if (maj_stat != GSS_S_COMPLETE)
	return maj_stat;

    if (recv_tok.length < sizeof(gsssasl_security_negotiation_desc)) {
	gss_release_buffer(&rc, &recv_tok);
	return GSS_S_FAILURE;
    }
    /*
     * we're lazy and don't copy the token. This could cause
     * problems if libgssapi uses a different malloc!
     */
    *masklength = recv_tok.length;
    *outmask = (gsssasl_security_negotiation_t) recv_tok.value;
    if (*outmask == NULL) {
	gss_release_buffer(&rc, &recv_tok);
	return GSS_S_FAILURE;
    }
/*XXX
   (*outmask)->token_size = ntohl((*outmask)->token_size);
 */
    return GSS_S_COMPLETE;
}

#endif				/* _GSSSASL_H_ */
