/*--

THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED
TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
PARTICULAR PURPOSE.

Copyright (C) 1999  Microsoft Corporation.  All rights reserved.

Module Name:

    locate_kpasswd.c

Abstract:

    Locate the kpasswd service

--*/
/*
 * lib/krb5/os/locate_kdc.c
 *
 * Copyright 1990 by the Massachusetts Institute of Technology.
 * All Rights Reserved.
 *
 * Export of this software from the United States of America may
 *   require a specific license from the United States Government.
 *   It is the responsibility of any person or organization contemplating
 *   export to obtain such a license before exporting.
 * 
 * WITHIN THAT CONSTRAINT, permission to use, copy, modify, and
 * distribute this software and its documentation for any purpose and
 * without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both that copyright notice and
 * this permission notice appear in supporting documentation, and that
 * the name of M.I.T. not be used in advertising or publicity pertaining
 * to distribution of the software without specific, written prior
 * permission.  M.I.T. makes no representations about the suitability of
 * this software for any purpose.  It is provided "as is" without express
 * or implied warranty.
 * 
 *
 * get socket addresses for KDC.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
#define NEED_SOCKETS
#include "k5-int.h"
#include <stdio.h>
#include <arpa/nameser.h>
#include <resolv.h>
#include "kpasswd.h"
#include "locator.h"

#ifndef KRB5_USE_INET
#ifdef HAVE_NETINET_IN_H
#define KRB5_USE_INET 1
#endif
#endif

static const char rcsid[] = "$Id$";

/*
 * returns count of number of addresses found
 */
krb5_error_code
krb5_locate_kpasswd_conf(context, realm, addr_pp, naddrs)
    krb5_context context;
    const krb5_data *realm;
    struct sockaddr **addr_pp;
    int *naddrs;
{
    const char	*realm_kdc_names[4];
    char **hostlist, *host, *port, *cp;
    krb5_error_code code;
    int i, j, out, count;
    struct sockaddr *addr_p;
    struct sockaddr_in *sin_p;
    struct hostent *hp;
    struct servent *sp;
#ifdef KRB5_USE_INET
    u_short udpport = htons(DEFAULT_KPASSWD_PORT);
#endif

    if ((host = malloc(realm->length + 1)) == NULL) 
	return ENOMEM;

    strncpy(host, realm->data, realm->length);
    host[realm->length] = '\0';
    hostlist = 0;
    
    realm_kdc_names[0] = "realms";
    realm_kdc_names[1] = host;
    realm_kdc_names[2] = "kpasswd_server";
    realm_kdc_names[3] = 0;

    code = profile_get_values(context->profile, realm_kdc_names, &hostlist);

    if (code == PROF_NO_SECTION)
	return KRB5_REALM_UNKNOWN;
    if (code == PROF_NO_RELATION)
	return KRB5_CONFIG_BADFORMAT;
    if (code)
	return code;

#ifdef KRB5_USE_INET
    if ((sp = getservbyname("kpasswd", "udp")))
	udpport = sp->s_port;
#endif
    count = 0;
    while (hostlist && hostlist[count])
	    count++;
    
    if (count == 0) {
	*naddrs = 0;
	return 0;
    }
    
    addr_p = (struct sockaddr *)malloc (sizeof (struct sockaddr) * count);
    if (addr_p == NULL)
	return ENOMEM;

    for (i=0, out=0; hostlist[i]; i++) {
	host = hostlist[i];
	/*
	 * Strip off excess whitespace
	 */
	cp = strchr(host, ' ');
	if (cp)
	    *cp = 0;
	cp = strchr(host, '\t');
	if (cp)
	    *cp = 0;
	port = strchr(host, ':');
	if (port) {
	    *port = 0;
	    port++;
	}
	hp = gethostbyname(hostlist[i]);
	if (hp != 0) {
	    switch (hp->h_addrtype) {
#ifdef KRB5_USE_INET
	    case AF_INET:
		for (j=0; hp->h_addr_list[j]; j++) {
		    sin_p = (struct sockaddr_in *) &addr_p[out++];
		    memset ((char *)sin_p, 0, sizeof(struct sockaddr));
		    sin_p->sin_family = hp->h_addrtype;
		    sin_p->sin_port = port ? htons(atoi(port)) : udpport;
		    memcpy((char *)&sin_p->sin_addr,
			   (char *)hp->h_addr_list[j],
			   sizeof(struct in_addr));
		    if (out+1 >= count) {
			count += 5;
			addr_p = (struct sockaddr *)
			    realloc ((char *)addr_p,
				     sizeof(struct sockaddr) * count);
		    }
		}
		break;
#endif
	    default:
		break;
	    }
	}
	free(hostlist[i]);
	hostlist[i] = 0;
    }
    free ((char *)hostlist);

    if (out == 0) {     /* Couldn't resolve any KDC names */
        free (addr_p);
        return KRB5_REALM_CANT_RESOLVE;
    }

    *addr_pp = addr_p;
    *naddrs = out;
    return 0;
}

krb5_error_code
krb5_locate_dns_srv(context, realm, service, protocol, addr_pp, naddrs)
    krb5_context context;
    const krb5_data *realm;
    const char *service;
    const char *protocol;
    struct sockaddr **addr_pp;
    int *naddrs;
{
    krb5_error_code code;
    int len, out, j, count;
    unsigned char reply[1024];
    struct sockaddr *addr_p = NULL;
    struct sockaddr_in *sin_p;
    struct hostent *hp;
    unsigned char *p;
    char host[128];
    int status;
    int priority, weight;
    u_short port;
    
    out = 0;
    addr_p = (struct sockaddr *)malloc (sizeof (struct sockaddr));
    if (addr_p == NULL)
	return ENOMEM;

    count = 1;
    
    /* XXX need to change this to call locate_dns() */

#ifdef HAVE_SNPRINTF
    snprintf(host, sizeof(host), "%s.%s.%*s.",
	     service, protocol, realm->length, realm->data);
#else
    sprintf(host, "%s.%s.%*s.",
	     service, protocol, realm->length, realm->data);
#endif

    /* printf("krb5_locate_dns_srv: locate %s\n", host); */

    len = res_search(host, C_IN, T_SRV, reply, sizeof(reply));
    if (len >=0) {

	/* Parse out query */
	p = reply;
	p += sizeof(HEADER);
	status = dn_expand(reply, reply + len, p, host, sizeof(host));
	if (status < 0) {
	    goto out;
	}
	p += status;
	p += 4;

	while (p < reply + len) {
	    int type, class, ttl, size;
	    status = dn_expand(reply, reply + len, p, host, sizeof(host));
	    if (status < 0) {
		goto out;
	    }
	    p += status;
	    type = (p[0] << 8) | p[1];
	    p += 2;
	    class = (p[0] << 8) | p[1];
	    p += 2;
	    ttl = (p[0] << 24) | (p[1] << 16) | (p[2] << 8) | p[3];
	    p += 4;
	    size = (p[0] << 8) | p[1];
	    p += 2;
	    if (type == T_SRV) {
		status = dn_expand(reply, reply + len, p + 6, host, sizeof(host));
		if (status < 0) {
		    goto out;
		}
		priority = (p[0] << 8) | p[1];
		weight = (p[2] << 8) | p[3];
		port = (p[4] << 8) | p[5];

		/* printf("%d: %s:%d [%d:%d]\n", out, host, port, weight, priority); */
		/* XXX This should really be sorted by priority */
		hp = gethostbyname(host);
		if (hp != 0) {
		    switch (hp->h_addrtype) {
#ifdef KRB5_USE_INET
		    case AF_INET:
			for (j=0; hp->h_addr_list[j]; j++) {
			    sin_p = (struct sockaddr_in *) &addr_p[out++];
			    memset ((char *)sin_p, 0, sizeof(struct sockaddr));
			    sin_p->sin_family = hp->h_addrtype;
			    sin_p->sin_port = htons(port);
			    memcpy((char *)&sin_p->sin_addr,
				   (char *)hp->h_addr_list[j],
				   sizeof(struct in_addr));
			    if (out+1 >= count) {
				count += 5;
				addr_p = (struct sockaddr *)
				    realloc ((char *)addr_p,
					     sizeof(struct sockaddr) * count);
				if (!addr_p)
				    goto out;
			    }
			}
			break;
#endif
		    default:
			break;
		    }
		}
		p += size;
	    }
	}
    }
    
out:
    if (out == 0) {     /* Couldn't resolve any KDC names */
	free (addr_p);
	return KRB5_REALM_CANT_RESOLVE;
    }

    *addr_pp = addr_p;
    *naddrs = out;
    return 0;
}

krb5_error_code
krb5_locate_kpasswd(context, realm, addr_pp, naddrs)
    krb5_context context;
    const krb5_data *realm;
    struct sockaddr **addr_pp;
    int *naddrs;
{
    krb5_error_code code;
    int i;
    
    /* Let the conf file override the DNS */
    code = krb5_locate_kpasswd_conf(context, realm, addr_pp, naddrs);

    if (code && getenv("KRB5_NO_SRV") == 0) {
	code = krb5_locate_dns_srv(context, realm, "_kpasswd", "_udp",
				   addr_pp, naddrs);
	if (code)
	    code = krb5_locate_dns_srv(context, realm, "_kpasswd", "_tcp",
				       addr_pp, naddrs);
    }

#if 0
    printf("%d addrs\n", *naddrs);
    for (i = 0; i < *naddrs; i++) {
	struct sockaddr_in *ap = (struct sockaddr_in *)&((*addr_pp)[i]);
	printf("%d: %d %s %d\n", i, ap->sin_family,
	       inet_ntoa(ap->sin_addr), ntohs(ap->sin_port));
    }
#endif

    return(code);
}
